/*
 * Copyright © 2011 David King <amigadave@amigadave.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <gst/gst.h>
#ifdef GDK_WINDOWING_X11
  #include <X11/Xlib.h>
#endif
#include <clutter-gtk/clutter-gtk.h>

#include "cheese-gtk.h"
#include "cheese.h"

/**
 * SECTION:cheese-gtk-init
 * @short_description: Initialize libcheese-gtk
 * @stability: Unstable
 * @include: cheese/cheese-gtk.h
 *
 * Call cheese_gtk_init() to initialize libcheese-gtk.
 */


/**
 * cheese_gtk_init:
 * @argc: pointer to the argument list count
 * @argv: pointer to the argument list vector
 *
 * Initialize libcheese-gtk, by initializing Clutter, GStreamer and GTK+. This
 * automatically calls cheese_init(), initializing libcheese.
 *
 * Returns: %TRUE if the initialization was successful, %FALSE otherwise
 */
gboolean
cheese_gtk_init (int *argc, char ***argv)
{
    ClutterInitError error;

#ifdef GDK_WINDOWING_X11
    XInitThreads ();
#endif

    error = gtk_clutter_init (argc, argv);

    if (error != CLUTTER_INIT_SUCCESS)
        return FALSE;

    if (!cheese_init (argc, argv))
        return FALSE;

    return TRUE;
}

/**
 * cheese_gtk_init_with_args:
 * @argc: pointer to the argument list count
 * @argv: pointer to the argument list vector
 * @parameter_string:  string which is displayed in the first line of --help
 * output, after programname [OPTION...]
 * @entries: a NULL-terminated array of GOptionEntries describing the options
 * of your program
 * @translation_domain: a translation domain to use for translating the
 * --help output for the options in entries with gettext(), or NULL
 * @error: a return location for errors
 *
 * Initialize libcheese-gtk, by initializing Clutter, GStreamer and GTK+. This
 * automatically calls cheese_init_with_args(), initializing libcheese.
 *
 * Returns: %TRUE if the initialization was successful, %FALSE otherwise
 */
gboolean
cheese_gtk_init_with_args (int *argc, char ***argv,
                           const char *parameter_string,
                           GOptionEntry *entries,
                           const char *translation_domain,
                           GError **error)
{
    GOptionContext *context;
    gboolean res;

#ifdef GDK_WINDOWING_X11
    /* We can't call clutter_gst_init() before gtk_clutter_init(), so no
     * choice but to initialise X11 threading ourself */
    XInitThreads ();
#endif

    /* We cannot simply call gtk_clutter_init_with_args() here, since that
     * will result in the commandline being parsed without gst support. */
    context = g_option_context_new (parameter_string);
    g_option_context_add_main_entries (context, entries, translation_domain);
    g_option_context_set_translation_domain (context, translation_domain);
    g_option_context_add_group (context, gst_init_get_option_group ());
    g_option_context_add_group (context, gtk_get_option_group (TRUE));
    g_option_context_add_group (context, cogl_get_option_group ());
    g_option_context_add_group (context,
        clutter_get_option_group_without_init ());
    g_option_context_add_group (context, gtk_clutter_get_option_group ());

    res = g_option_context_parse (context, argc, argv, error);

    g_option_context_free (context);

    if (!res)
        return FALSE;

    return cheese_init_with_args (argc, argv, parameter_string, entries,
                                  translation_domain, error);
}
